; ----------------------------------------------------------------------
;  DMX to RGB interface                    Vladimir  Visnovsky (C) 2012
; ----------------------------------------------------------------------

; Nacitanie kniznice pre AtMegu8
 .Include "wMega8.asm"

; Inicializacia ----------------------------------------------------- 1.
; - Inicializacia USART
;   - Nastavenie na UBRRL na 3 pri frekvencii 16Mhz je rychlost 250Kb 
;   - Povolenie prijimania a povolenie prerusenia od prijateho bytu
;   - 8.bit prenos s 1 Stopom
 Usart_Init

; - Inicializacia TIMERu
;   - f = 2Mhz  / HL		->		t = 0,5us x HL
;   - Casovac pre generovanie signalu PWM
;   - Nastavenie PWM frekvencie <<<<<<<<<<<<<<<<<<<<<<< Set PWM Speed
 Ldi	H		, 0x00
 Ldi	L		, 0x40
 Timer_Init

; - Inicializacia 8Bit TIMERu
;   - f = 2Mhz  / L		->		t = 0,5us x HL
;   - Casovac pre generovanie signalu Stroboskop
;   - Nastavenie stroboskop frekvencie <<<<<<<<<<<<< Set Strobo Speed
 ;Ldi	L		, 0x80
 ;Timer8_Init
 ;Timer8_Div1026

; Premenne a konstanty ---------------------------------------------- 2.
; - Nastavenie Maximalnej dlzky signalu DMX <<<<<<<<<<<<<< Length Dmx
;   - od 1 do 512
 Ldi	YH		, 0x01		; Horny Byte
 Ldi	YL		, 0xF0		; Dolny Byte
; - Pocitadlo prijatych bytov
;   - Vynulovanie
 Clr	ZH					; Horny Byte
 Clr	ZL					; Dolny Byte 
; - Nastavenie Adresi <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< Set Addres
;   - Nacitanie z EEPROM
;     - Vynulovanie HL - adresa EEPROM  
 Clr	H
 Clr	L
 EEPROM_Read
 Mov	XL		, A 
 Inc	L
 EEPROM_Read
 Mov	XH		, A 
 ; Ak je adresa mimo rozsah
 Cp		XL		, YL
 Cpc	XH		, YH
  Brlo	EEPROM_Loaded
 ; Ak je mimo rozsah, nacitaj prvu adresu			   
 Ldi	XH		, 0x00
 Ldi	XL		, 0x01
 ; Platne data EEPROM
 EEPROM_Loaded:
; - Registre otienov farieb
;   - Priradenie
 .Def	cR		= R20		; Jas Cervenej
 .Def	cG		= R21		; Jas Zelenej
 .Def	cb		= R22		; Jas Moderj
 .Def	cC		= R23		; Jas Pocitadlo 
;   - Vynulovanie 
 Ser	cR
 Ser	cG
 Ser	cB
 Clr	cC
; - Pomenovanie Pinov
;   - Piny pre statove LED
 .Equ	pSignal = 0
 .Equ	pError	= 1
 .Equ	pStore	= 2
;   - Piny tlacidiel
 .Equ 	pButton	= 5 
; - Registre Stroba
;  R0 - Dlzka medzery
;  R1 - Jas/ zloska Y
 .Def	sLength	= R0
 .Def	cY		= R1
;  sMask - Maska pre strobo FF - Svieti / 00 - Nesvieti
 .Def	sMask	= R2
;  sCount - Pocitadlo pre dlzku  
 .Def	sCount	= R3
;   - Cistenie regisrtov
 Ser	T
 Mov 	sMask	, T
 Clr	T
 Mov	sCount	, T
;   - YL - Pocitadlo na 6 -> pri 61Hz / 6 = 0,1s pre strobo
 Clr	YL   
; - Prednastavenie operacnych registrov
;   - Krok prijimaca
;     0 : Hladam signal
;     1 : Signal najdeny, cakam na start byte
;     2 : Start byte v poriadku, cakam na adresu
;     3 : Platne data R
;     4 : Platne data G
;     5 : Platne data B
 Clr	F 
 
; Spustenie
 Sei

; Hlavny Program ---------------------------------------------------- 4.
App_Begin:
 Nop
 RJmp	App_Begin

; Obsluha prerusenia ------------------------------------------------ 5.
; - USART : Prerusenie od prtijatia RX
;   - Nacitavame a dekodujeme DMX signal
USART_Interupt:
 PushA
 ; Nacitanie registrov USART
 ; - Nacitanie statusu
 In		B		, UCSRA
 ; - Nacitanie prichodzieho bytu
 In		A		, UDR
 ; Rozvetvenie podla statusu USART
 ; - vyhlasenie chyby FrameError (Pre nas signal, ze sme v BREAK pauze)
 Sbrc	B		, FE
  Rjmp	USART_Interupt_FE
 ; - vyhlasenie chyby DataOverRun
 Sbrc	B		, DOR
  Rjmp	USART_Interupt_DOR
 ; - Nacitanie prichodzieho bytu bolo bez chyby
 ; Rozvetvenie podla kroku F
 ; - Hladanie BREAK pauzi
 Cpi	F		, 0
  Breq	USART_Interupt_End
 ; - Overenie Start bytu
 Cpi	F		, 1
  Breq	USART_Interupt_StartByte
 ; - Hladanie Adresi
 Cpi	F		, 2
  Breq	USART_Interupt_FindAdres 
 ; - Toto Ncitanie je nadviazane na hladanie adresi!!! 
 ; - Nacitanie 1. bytu platnych dat
 ;   - Cpi	F		, 3	
 ;   -  Breq	USART_Interupt_LoadR
 ; - Nacitanie 2. bytu platnych dat
 Cpi	F		, 3
  Breq	USART_Interupt_LoadG
 ; - Nacitanie 3. bytu platnych dat
 Cpi	F		, 4
  Breq	USART_Interupt_LoadB
 ; - Nacitanie 4. bytu platnych dat
 Cpi	F		, 5
  Breq	USART_Interupt_LoadR0
 ; - Nacitanie 5. bytu platnych dat
 Cpi	F		, 6
  Breq	USART_Interupt_LoadR1
 ; Vynulovanie DMX pocitadla
 Clr 	F
 ; Koniec prerusenia 
 USART_Interupt_End:
  PopA 
  Reti
 ; Odskoky 						 
 ; - Status FE
 USART_Interupt_FE:				; -------------------- USART_Interupt_FE
  ; Sme v BREAK pauze. Budeme cakat na STARTbyte a overime platnost
  Ldi	F		, 1 
  ; Vynulujeme status USARTu
  USART_ResetFlag_FrameError	
 RJmp	USART_Interupt_End
 ; - Status DOR
 USART_Interupt_DOR:			; ------------------- USART_Interupt_DOR
  ; Budeme hladat zaciatok vysielania
  Clr	F		
  ; Vynulovanie statusu USARTu
  USART_ResetFlag_DataOverRun
  ; Zasvietenie LED Chyby
  Sbi	PortC	, pError
  ; Zhasenie LED Prijimu
  Cbi	PortC	, pSignal
 RJmp	USART_Interupt_End
 ; - Overenie StartBytu
 USART_Interupt_StartByte:		; ------------- USART_Interupt_StartByte
  ; Porovnanie nacitaneho bytu s nulov
  Cpi	A		, 0
   ; Ak start byt je iny ako 0, vyhlas chybu
   Brne	USART_Interupt_DOR
  ; Ak je start byt v poriadku
  ; - Zhasenie LED Chyby
  Cbi	PortC	, pError
  ; - Zasvietenie LED Prijimu
  Sbi	PortC	, pSignal
  ; - Pozdvihnutie Kroku na hladanie adresy
  Inc	F
  ; Vynulovanie pocitadla prichodzich bytov
  Clr	ZH
  Clr	ZL
 RJmp	USART_Interupt_End
 ; - Hladanie adresy
 USART_Interupt_FindAdres:		; ------------- USART_Interupt_FindAdres  
  ; Pripocitanie ko pocitadlu pryjimanych bytov 
  Adiw	Z		, 1
  ; Kontrola stlacenim tlacidla
  Sbis	PinC	, pButton
   RJmp	USART_Interupt_Button
   ; Zhasnutie Led  
   Cbi	PortC	, pStore
  ; Porovnanie Adresi s pocitadlom prijatych bytov
  Cp	ZL		, XL
  Cpc	ZH		, ZH
   ; Adresa nenajdena
   Brne 	USART_Interupt_End
   ; Adresa Najdena
   ; - Hned nacitame prvy udaj 
 ; - Nacitanie platnych dat
 ;   - Cervena Farba
 USART_Interupt_LoadR:			; ----------------- USART_Interupt_LoadR
  Mov	cR		, A
  ; Zlozka Y. Ak je Farba > cY, Farba:=cY
  Cp	cR		, cY
   Brlo USART_Interupt_LoadR_Y
   Mov	cR		, cY
  USART_Interupt_LoadR_Y:
  ; Krok DMX prijimaca na dalsi byte 
  Inc	F
 RJmp	USART_Interupt_End
 ;   - Zelena Farba
 USART_Interupt_LoadG:			; ----------------- USART_Interupt_LoadG
  Mov	cG		, A
  ; Zlozka Y. Ak je Farba > cY, Farba:=cY
  Cp	cG		, cY
   Brlo USART_Interupt_LoadG_Y
   Mov	cG		, cY
  USART_Interupt_LoadG_Y:
  ; Krok DMX prijimaca na dalsi byte 
  Inc	F
 RJmp	USART_Interupt_End
 ;   - Modra Farba
 USART_Interupt_LoadB:			; ----------------- USART_Interupt_LoadB 
  Mov	cB		, A
  ; Zlozka Y. Ak je Farba > cY, Farba:=cY
  Cp	cB		, cY
   Brlo USART_Interupt_LoadB_Y
   Mov	cB		, cY
  USART_Interupt_LoadB_Y:
  ; Krok DMX prijimaca na dalsi byte 
  Inc	F
 RJmp	USART_Interupt_End
 USART_Interupt_LoadR0:			; ---------------- USART_Interupt_LoadR0
  Mov	cY		, A
  Inc	F
 RJmp	USART_Interupt_End
 USART_Interupt_LoadR1:			; ---------------- USART_Interupt_LoadR1 
  Mov	sLength		, A
  Inc	F
 RJmp	USART_Interupt_End
 ; Ulozenie Adresi
 USART_Interupt_Button:			; ---------------- USART_Interupt_Button
  ; Najdenie bytu, ktory je > ako 128
  Cpi	A		, 128
   Brlo USART_Interupt_End
  ; Ak sme nasli byte, ktory je vyssi ako 128
  ; - Rozsvietime LED
  Sbi	PortC	, pStore
  ; - Ulozime ho do X  
  Mov	XH		, ZH
  Mov	XL		, ZL
  ; - Ulozenie do EEPROM
  Clr 	L
  Mov 	A		, XL
  EEPROM_Write
  Inc 	L
  Mov 	A		, XH
  EEPROM_Write
  ; Krokovac DMX dekodera na zaciatok
  Clr	F
  Rjmp USART_Interupt_End
  
; - Timer : Prerusenie od pretecenia casovaca
;   - Generujeme PWM signaly pre RGB Led
Timer_Interupt:
 PushA
  ; Nacitanie hodnoty portu
  In	B		, PortD
  ; Inkrement pocitadla					
  Inc 	cC										; -+ Zaciatok PWM
  ; Ak neprejde nulov							;  | - Rozsvietenie LED
  Cpi	cC		, 0    							;  | - Generator 0,1s
   Brne Timer_Interupt_CNoZero					;  |
   ; Ak prejde nulov							;  |
   ; - Rozsvietenie LEDiek						;  |
   Ori	B		, 0b11100000					;  |
   ; - Skok na obsluhu Stroba pri 6tej slucke	;  |
   Inc	YL										;  |
   Cpi	YL		, 2								;  |
    ; Ak je siesta slucka						;  |
    Breq   Timer_Interupt_Strobo				;  |
 Timer_Interupt_CNoZero:						; -+
  ; Prva Farba
  ; - Porovnanie registra farby z pocitadlom PWM
  Cp	cR		, cC	
   Brne	Timer_Interupt_OverRed
   ; Zhasenie prislusneho vystupu
   Andi	B		, 0b11011111
  Timer_Interupt_OverRed: 
  ; Druha Farba
  ; - Porovnanie registra farby z pocitadlom PWM
  Cp	cG		, cC	
   Brne	Timer_Interupt_OverGreen
   ; Zhasenie prislusneho vystupu
   Andi	B		, 0b10111111
  Timer_Interupt_OverGreen: 
  ; Tretia Farba
  ; - Porovnanie registra farby z pocitadlom PWM
  Cp	cB		, cC	
   Brne	Timer_Interupt_OverBlue
   ; Zhasenie prislusneho vystupu
   Andi	B		, 0b01111111
  Timer_Interupt_OverBlue: 
  ; Maska strobo
  And	B		, sMask
  ; Vystup na port
  Out	PortD	, B 
 PopA
 Reti
 ; Obsluha strobo
 Timer_Interupt_Strobo: 			; ------------ Timer_Interupt_Strobo
  ; Vynulovat YL (pocitadla 0,1s)
  Clr	YL
  ; Negacia potencimetra rychlosti, konverzia rychlost -> cas
  Mov	YH		, sLength
  Com	YH
  ; Cela dlzka potenciometra deleno 8
  Lsr	YH
  Lsr	YH
  Lsr	YH
  ; Ak je sabla v nule, ignorujeme Strobo
  Cpi	YH		, 0
   Breq	Timer_Interupt_Strobo_Light
  ; Pocitadlo
  Inc	sCount
  Ldi	T		, 1	; po prvom cykle je uz sCount 1 a nie 0  
  ; Prvy cyklus
  Cp	sCount	, T
   Breq	Timer_Interupt_Strobo_Light
  ; Cyklus pocitadla  
  Cp	sCount	, YH
   Brlo Timer_Interupt_Strobo_Black
   ; Vynulovanie pocitadla
   Clr	T
   Mov	sCount	, T     
   RJmp	Timer_Interupt_CNoZero  
 ; Zhasenie Led -> Maska na FF
 Timer_Interupt_Strobo_Black:		; ------ Timer_Interupt_Strobo_Black
  ; Maska na 00
  Clr	T
  Mov	sMask	, T
 RJmp	Timer_Interupt_CNoZero
 ; Rozsvietenie Led -> Maska na FF
 Timer_Interupt_Strobo_Light:		; ------ Timer_Interupt_Strobo_Light
  ; Maska na FF
  Ser	T
  Mov	sMask	, T
 RJmp	Timer_Interupt_CNoZero
 
